﻿using System;
using System.Collections.Generic;
using System.IO;
using System.Linq;
using System.Runtime.InteropServices.WindowsRuntime;
using Windows.Foundation;
using Windows.Foundation.Collections;
using Windows.UI.Xaml;
using Windows.UI.Xaml.Controls;
using Windows.UI.Xaml.Controls.Primitives;
using Windows.UI.Xaml.Data;
using Windows.UI.Xaml.Input;
using Windows.UI.Xaml.Media;
using Windows.UI.Xaml.Navigation;

using System.Diagnostics;
using Windows.UI.Core;
using Windows.System;

// The Blank Page item template is documented at http://go.microsoft.com/fwlink/?LinkId=234238

namespace InsuranceApp
{
    /// <summary>
    /// Insurance Calculator Page
    /// </summary>
    public sealed partial class CalcPage : Page
    {
        ServerCommunication com;
        EventListener listener;
        public CalcPage()
        {
            this.InitializeComponent();
            com = new ServerCommunication();
            // Event listener for recieving info from async functions
            listener = new EventListener(com, this);

            // Set initial values
            // buildYear
            this.calc_buildYear_dp.MaxYear = DateTimeOffset.Now;
            this.calc_buildYear_dp.MinYear = DateTimeOffset.Parse("1000.01.01");
            // StartDate
            this.calc_insuranceStartDate_dp.MinYear = DateTimeOffset.Now;
            // validTo
            this.calc_validTo_dp.MinYear = DateTimeOffset.Now;

            // Hide Ordering related things
            this.calc_name_tb.Visibility = Visibility.Collapsed;
            this.calc_surName_tb.Visibility = Visibility.Collapsed;
            this.calc_order_btn.Visibility = Visibility.Collapsed;
            this.calc_validTo_dp.Visibility = Visibility.Collapsed;
            this.calc_order_btn.Visibility = Visibility.Collapsed;
            this.calc_orderConfirm_btn.Visibility = Visibility.Collapsed;
            this.calc_delete_btn.Visibility = Visibility.Collapsed;
        }

        // Show price on page
        public void setPrice(string price)
        {
            this.calc_price_tb.Text = price;
        }

        // On confirmation
        public void orderConfirmation(string id)
        {
            this.calc_notificationBox_tb.Text = "Confirmed: " + id;
        }
        // On delete
        public void orderDeleted()
        {
            this.calc_notificationBox_tb.Text = "Application deleted";
        }

        // back button
        private void calc_back_btn_Click(object sender, RoutedEventArgs e)
        {
            com = null;
            listener = null;
            this.Frame.Navigate(typeof(MainPage));
        }

        // Calculate button
        private void calc_calc_btn_Click(object sender, RoutedEventArgs e)
        {
            string area = String.Empty;
            int postalCode_i = 0;
            double area_d = 0.0;
            bool input_okay = true;
            string bad_input = "Bad input";

            // Read Postal Code
            try
            {
                postalCode_i = Convert.ToInt32(calc_postalCode_tb.Text);
            }
            catch (FormatException)
            {
                calc_postalCode_tb.Text = bad_input;
            }
            catch (OverflowException)
            {
                calc_postalCode_tb.Text = bad_input;
            }
            string postalCode = postalCode_i.ToString();

            // Read Address
            string address = this.calc_address_tb.Text;

            // Read Area
            try
            {
                area_d = Convert.ToDouble(calc_area_tb.Text);
            }
            catch (FormatException)
            {
                calc_area_tb.Text = bad_input;
            }
            catch (OverflowException)
            {
                calc_area_tb.Text = bad_input;
            }
            area = String.Format("{0:0.0}", area_d);

            // Read Build Year
            string buildYear = this.calc_buildYear_dp.Date.ToString("yyyy"); ;

            // Read Start Date
            string insuranceStartDate = this.calc_insuranceStartDate_dp.Date.ToString("dd.MM.yyyy");

            // Read House Type
            string houseType = this.calc_houseType_cb.SelectedValue.ToString();

            // Read Billing Period
            string billingPeriod = this.calc_billingPeriod_cb.SelectedValue.ToString();

            // Read Currency
            string currency = this.calc_currency_cb.SelectedValue.ToString();

            // Check for empty input
            if (String.IsNullOrEmpty(postalCode) || postalCode_i == 0)
            {
                input_okay = false;
                this.calc_postalCode_tb.Text = bad_input;
            }
            if (String.IsNullOrEmpty(address))
            {
                input_okay = false;
                this.calc_address_tb.Text = bad_input;
            }
            if (String.IsNullOrEmpty(area))
                input_okay = false;

            if (input_okay)
            {
                com.saveData(postalCode, address, area, buildYear, insuranceStartDate, houseType, billingPeriod, currency);

                com.calculateInsurance();

                // Show Order button
                this.calc_order_btn.Visibility = Visibility.Visible;
            }
        }

        // Order confirm button
        private void calc_orderConfirm_btn_Click(object sender, RoutedEventArgs e)
        {
            // Read order info
            string name = this.calc_name_tb.Text;
            string surName = this.calc_surName_tb.Text;
            string validTo = this.calc_validTo_dp.Date.ToString("dd.MM.yyyy");
            // Hopefully place order
            com.orderInsurance(name, surName, validTo);
            // Show delete button
            this.calc_delete_btn.Visibility = Visibility.Visible;
        }

        // Order button
        private void calc_order_btn_Click(object sender, RoutedEventArgs e)
        {
            // Make order info boxes and confirm button visible
            this.calc_name_tb.Visibility = Visibility.Visible;
            this.calc_surName_tb.Visibility = Visibility.Visible;
            this.calc_orderConfirm_btn.Visibility = Visibility.Visible;
            this.calc_validTo_dp.Visibility = Visibility.Visible;
            this.calc_order_btn.Visibility = Visibility.Collapsed;
        }

        // Clear textbox on select
        private void calc_textBox_GotFocus(object sender, RoutedEventArgs e)
        {
            TextBox tb = (TextBox)sender;
            tb.Text = string.Empty;
            tb.GotFocus -= calc_textBox_GotFocus;
        }

        // Delete button
        private void calc_delete_btn_Click(object sender, RoutedEventArgs e)
        {
            // Delete application from remote server
            com.deleteInsurance();
        }

        private void calc_postalCode_tb_KeyDown(object sender, KeyRoutedEventArgs e)
        {
            char key = Convert.ToChar(e.Key);

            //if the key is not 0-9 prevent the event from being handled further
            if (!(key >= '0' && key <= '9'))
                e.Handled = true; 
        }

        private void calc_area_tb_KeyDown(object sender, KeyRoutedEventArgs e)
        {
            char key = Convert.ToChar(e.Key);

            //if the key is not 0-9 or . prevent the event from being handled further
            if ((key >= '0' && key <= '9') || (key == '.'))
                e.Handled = false;
            else
                e.Handled = true;

        }
    }
}
